package ApplicationLayer;

import ApplicationLayer.Circuit.Circuit;
import PresentationLayer.*;
import PresentationLayer.Forms.MainForm;
import PresentationLayer.Grid.CircuitGrid;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import org.apache.commons.math.linear.SingularMatrixException;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class ApplicationLayer implements IApplicationLayer {

    public static ApplicationLayer self = new ApplicationLayer();
    private IPresentationLayer pl;
    private CircuitGrid cg;
    private Circuit c;
    private Equations eqs;

    private ApplicationLayer() {
    }

    /**
     * spuštění a inicializace
     */
    public void run() {
        cg = CircuitGrid.self;
        eqs = Equations.self;
        pl = new MainForm();
    }

    /**
     * zpracuje obvod, vytvoří a vypočítá rovnice (aby se zjistilo, zda je správně navržen)
     * nechá zobrazit prvky obvodu
     */
    public void processCG() {
        cg.setMarks();
        c = ConvertToLogic.self.convert();
        try {
            c.markBranches();
            c.generateLoopSystems();
            eqs.generateEquations(c);
            eqs.count(c);
            pl.showBranches(c);
        } catch (IndependentCircuitsException e) {
            pl.showErrorMessage("Obvod se pravděpodobně skládá z více nezávislých obvodů a nemá smysl jej počítat jako celek.");
        } catch (ShortCircuitException e) {
            pl.showErrorMessage("Obvod je zkratovaný nebo obsahuje paralelní prázdné větve!\nObvod obsahuje smyčku s nulovým odporem a nemá smysl jej počítat.\nOdstraňte přebytečné prázdné větve nebo doplňte chybějící odpory.");
        } catch (EmptyLoopException e) {
            pl.showErrorMessage("Obvod se pravděpodobně skládá z více nezávislých obvodů a nemá smysl jej počítat jako celek.");
        } catch (SingularMatrixException e) {
            pl.showErrorMessage("Obvod se pravděpodobně skládá z více nezávislých obvodů a nemá smysl jej počítat jako celek.");
        }
    }

    /**
     * nechá zobrazit rovnice
     */
    public void generateEquations() {
        pl.showEquations(eqs.equations);
    }

    /**
     * nechá zobrazit řešení rovnic
     */
    public void solveEquations() {
        eqs.saveSolution(c);
        pl.showSolution(eqs.getSolution(), c.needInterpretSolution());
    }

    /**
     *  interpretuje výsledky (záporné a nulové hodnoty proudu)
     */
    public void interpretSolution() {
        c.interpretSolution();
        pl.interpretSolution();
    }

    public static String loadFileToString(String path) {
        InputStream in = null;
        String lines = "";
        try {
            in = MainForm.class.getResourceAsStream(path);
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String line = null;
            while ((line = reader.readLine()) != null) {
                lines += line;
                //System.out.println(line);
            }
        } catch (IOException ex) {
            System.err.println(ex);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ex) {
                }
            }
        }
        return lines;
    }
}
