package ApplicationLayer.Circuit;

import ApplicationLayer.NumOp;
import ApplicationLayer.Position;
import ApplicationLayer.Direction;
import PresentationLayer.Grid.CircuitGrid;
import PresentationLayer.Grid.GFNode;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class Branch implements Highlightable {

    public ArrayList<Node> nodes = new ArrayList<Node>(2);
    public ArrayList<Direction> directions = new ArrayList<Direction>(2);//směry, kterými větev začíná a končí (které části uzlů patří k větvi)
    public ArrayList<Component> components = new ArrayList<Component>(8);
    private ArrayList<Position> positions = new ArrayList<Position>();
    public int current = 1;//směr proudu: 1 = od nodes.get(0) do nodes.get(1)
    private double I;
    private int mark;

    public Branch(Node nodeA, Direction dirA, Node nodeB, Direction dirB, ArrayList<Component> components, ArrayList<Position> positions) {
        nodes.add(nodeA);
        nodes.add(nodeB);
        directions.add(dirA);
        directions.add(dirB);
        this.components = components;
        this.positions = positions;
    }

    public boolean hasNode(Node node) {
        return (nodes.contains(node));
    }

    /**
     * @param node jeden uzel
     * @return ten druhý uzel/null, pokud ho neobsahuje
     */
    public Node otherNode(Node node) {
        if (hasNode(node)) {
            return (node == nodes.get(0)) ? nodes.get(1) : nodes.get(0);
        } else {
            return null;
        }
    }

    /**
     * @return směr větve od daného uzlu/null, pokud uzel neobsahuje
     */
    public Direction getDirection(Node node) {
        if (nodes.get(0).equals(node)) {
            return directions.get(0);
        }
        if (nodes.get(1).equals(node)) {
            return directions.get(1);
        }
        return null;
    }

    /**
     * @return označení smyčky - název do listu
     */
    @Override
    public String toString() {
        return ("Větev " + mark/* + "\n  " + nodes.get(0).toString() + " Dir: " + directions.get(0) + "\n Comp: " + components.size() + "\n  " + nodes.get(1).toString() + " Dir: " + directions.get(1) + "\n---\n"*/);
    }

    /**
     * @return pozice polí větve včetně uzlů
     */
    public ArrayList<Position> getPositions() {
        ArrayList<Position> poss = new ArrayList<Position>();
        poss.add(nodes.get(0).pos);
        poss.addAll(positions);
        poss.add(nodes.get(1).pos);
        return poss;
    }

    /**
     * @return pozice polí v pořadí od daného uzle (včetně daného uzle)
     */
    public ArrayList<Position> getPositions(Node fromNode) {
        ArrayList<Position> poss = new ArrayList<Position>();
        if (nodes.get(0).equals(fromNode)) {
            poss.add(nodes.get(0).pos);
            poss.addAll(positions);
        } else {
            poss.add(nodes.get(1).pos);
            for (int i = positions.size() - 1; i >= 0; i--) {
                poss.add(positions.get(i));
            }
        }
        return poss;
    }

    /**
     * @return celkový odpor větve (vždy kladný)
     */
    public double getResistance() {
        double resistance = 0;
        Iterator<Component> it = components.iterator();
        Component c;
        while (it.hasNext()) {
            c = it.next();
            if (!c.isSource) {
                resistance += c.resistance;
            }
        }
        return NumOp.roundDouble(resistance);
    }

    /**
     * @return součet napětí zdrojů ve větvi větvi (kladný, pokud + směřuje k nodes.get(1))
     */
    public double getVoltage() {
        double voltage = 0;
        Iterator<Component> it = components.iterator();
        Component c;
        while (it.hasNext()) {
            c = it.next();
            if (c.isSource) {
                voltage += (c.plusNode == nodes.get(1)) ? c.voltage : -c.voltage;
            }
        }
        return NumOp.roundDouble(voltage);
    }

    public Node getFirstNode() {
        return (current != -1) ? nodes.get(0) : nodes.get(1);
    }

    /**
     * @return the I
     */
    public double getI() {
        return I;
    }

    /**
     * @param I the I to set
     */
    public void setI(double I) {
        this.I = I;
    }

    /**
     * @return potřebuje změnit směr v nákresu?
     */
    public boolean needInterpret() {
        return (I <= 0);
    }

    /**
     * změní směry proudu v nákresu, aby odpovídaly kladé hodnotě proudu
     * odstraní směr proudu v nákresu, pokud proud větví neteče
     */
    public void interpretSolution() {
        if (I < 0) {
            current = -current;
            this.I *= -1;
            for (int i = 0; i <= 1; i++) {
                Node node = nodes.get(i);
                ((GFNode) CircuitGrid.self.get(node.pos)).invertCurrent(directions.get(i));
            }
        }
        if (I == 0) {
            current = 0;
            for (int i = 0; i <= 1; i++) {
                Node node = nodes.get(i);
                ((GFNode) CircuitGrid.self.get(node.pos)).setCurrent(directions.get(i), 0);
            }
        }
    }

    /**
     * @return the mark
     */
    public int getMark() {
        return mark;
    }

    /**
     * @param mark the mark to set
     */
    public void setMark(int mark) {
        this.mark = mark;
        nodes.get(0).node.setBranch(directions.get(0), mark);
        nodes.get(1).node.setBranch(directions.get(1), mark);
    }
}
