package ApplicationLayer.Circuit;

import ApplicationLayer.Position;
import ApplicationLayer.EmptyLoopException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class Loop implements Highlightable {

    public ArrayList<Branch> branches = new ArrayList<Branch>();
    public ArrayList<Node> nodes = new ArrayList<Node>();
    private ArrayList<Position> positions = new ArrayList<Position>();

    public Loop(ArrayList<Node> nodes, ArrayList<Branch> tree, Branch branch) throws EmptyLoopException {
        this.nodes = (ArrayList<Node>) nodes.clone();
        branches = (ArrayList<Branch>) tree.clone();
        branches.add(branch);
        while (removeExcessiveBranches()) {
        }
        if (branches.isEmpty()) {
            throw new EmptyLoopException();
        }
        order();
        setPositions();
    }

    /**
     * odstraní přebývající větev smyčky (while cyklus)
     * @return byla odstraněna větev?
     */
    private boolean removeExcessiveBranches() {
        Iterator<Node> it = nodes.iterator();
        Node node;
        boolean removed = false;
        while (!removed & it.hasNext()) {
            node = it.next();
            if (branches(node).size() < 2) {
                removeBranch(node);
                removed = true;
            }
        }
        //System.out.println("Remove excessive branches: " + removed + " Branches: " + branches.size());
        return removed;
    }

    /**
     * odstraní větev vedocí z výchozího uzlu (pouze jednu)
     */
    private void removeBranch(Node node) {
        Iterator<Branch> it = branches.iterator();
        Branch br;
        boolean removed = false;
        while (!removed && it.hasNext()) {
            br = it.next();
            if (br.hasNode(node)) {
                branches.remove(br);
                removed = true;
            }
        }
        nodes.remove(node);
    }

    /**
     * @param node výchozí uzel
     * @return větve vycházející z tohoto uzlu
     */
    private ArrayList<Branch> branches(Node node) {
        ArrayList<Branch> brs = new ArrayList<Branch>();
        Iterator<Branch> itBr = branches.iterator();
        while (itBr.hasNext()) {
            Branch br = itBr.next();
            if (br.hasNode(node)) {
                brs.add(br);
            }
        }
        return brs;
    }

    /**
     * @return označení smyčky - název do listu
     */
    @Override
    public String toString() {
        String s = "Smyčka ";
        Iterator<Node> itN = nodes.iterator();
        Iterator<Branch> itB = branches.iterator();
        while (itN.hasNext()) {
            s = s.concat(itN.next().node.getMark() + ",");
            s = s.concat(Integer.toString(itB.next().getMark()) + ",");
        }
        return (s.substring(0, s.length() - 1));
    }

    /**
     * uloží do listu positions pozice potřebné k zvýraznění smyčky
     */
    private void setPositions() {
        positions.clear();
        Iterator<Node> itN = nodes.iterator();
        Iterator<Branch> itB = branches.iterator();
        while (itN.hasNext()) {
            positions.addAll(itB.next().getPositions(itN.next()));
        }
    }

    /**
     * seřadí uzly a větve v listech aby odpovídaly pořadí při procházení smyčky
     */
    private void order() {
        ArrayList<Branch> list = (ArrayList<Branch>) branches.clone();
        Iterator<Branch> it;
        Branch br;
        branches.clear();
        branches.add(list.remove(0));
        nodes.clear();
        nodes.add(branches.get(0).nodes.get(0));
        int end = list.size();
        Node node = branches.get(0).nodes.get(1);
        for (int i = 1; i <= end; i++) {
            nodes.add(node);
            it = list.iterator();
            while (i == branches.size() && it.hasNext()) {
                br = it.next();
                if (br.hasNode(node)) {
                    branches.add(br);
                    list.remove(br);
                    node = br.otherNode(node);
                }
            }
        }
    }

    public ArrayList<Position> getPositions() {
        return positions;
    }
}
