package ApplicationLayer;

import ApplicationLayer.Circuit.Branch;
import ApplicationLayer.Circuit.Circuit;
import ApplicationLayer.Circuit.Component;
import ApplicationLayer.Circuit.Node;
import PresentationLayer.Grid.CircuitGrid;
import PresentationLayer.Grid.GFNode;
import PresentationLayer.Grid.GFResistor;
import PresentationLayer.Grid.GFSource;
import PresentationLayer.Grid.GFWire;
import PresentationLayer.Grid.GridField;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class ConvertToLogic {

    public static ConvertToLogic self = new ConvertToLogic();
    private CircuitGrid cg = CircuitGrid.self;
    private Circuit c;

    /**
     * převede oblod z mřížky do logické struktury
     */
    public Circuit convert() {
        c = new Circuit();
        Position pos = findNode();
        c.nodes.add(new Node(new Position(pos), (GFNode) cg.get(pos)));
        findBranches(c.nodes.get(0));
        Iterator<Node> it = c.nodes.iterator();
        while (it.hasNext()) {
            ((GFNode) cg.get(it.next().pos)).setKnowCurrents(true);
        }
        return c;
    }

    /**
     * vybere všechny dosud neznámé větve vycházející ze vstupního uzlu a načte je
     */
    private void findBranches(Node node) {
        if (node == null) {
            return;
        }
        ArrayList<Direction> dirs = getNode(node).getDirs();
        Iterator<Direction> it = dirs.iterator();
        Direction dir;
        while (it.hasNext()) {
            dir = it.next();
            if (!c.hasBranch(node, dir)) {
                findBranches(findBranch(node, dir));
            }
        }

    }

    /**
     * @return uzel, který je první na řadě při čtení mřížky po řádcích
     */
    private Position findNode() {
        for (int row = 0; row < cg.getRowCount(); row++) {
            for (int col = 0; col < cg.getColCount(); col++) {
                if (cg.get(row, col).getClass() == GFNode.class) {
                    return new Position(row, col);
                }
            }
        }
        return new Position(0, 0);
    }

    /**
     * projde políčko po políčku větev určenou vstupním uzlem a směrem
     * načte novou větev a zvolí směr proudu
     * načte nový uzel (pokud najde)
     * @return nový uzel, pokud uzel na konci hledané větve není nový, vrací null
     */
    private Node findBranch(Node startNode, Direction startDir) {
        Position pointer = new Position(startNode.pos);
        ArrayList<Position> positions = new ArrayList<Position>();//seznam políček větve
        ArrayList<Component> components = new ArrayList<Component>();//seznam komponent větve
        pointer = move(startDir, pointer);
        Direction lastDir = startDir;
        while (cg.get(pointer).getClass() != GFNode.class) {
            positions.add(new Position(pointer));
            if (cg.get(pointer).getClass() == GFResistor.class) {
                components.add(new Component(((GFResistor) cg.get(pointer)).getResistance()));
            }
            if (cg.get(pointer).getClass() == GFSource.class) {
                components.add(new Component(((GFSource) cg.get(pointer)).getVoltage(), ((((GFSource) cg.get(pointer)).getPolarity() & (lastDir.ordinal() == 1 | lastDir.ordinal() == 2)) | (!((GFSource) cg.get(pointer)).getPolarity() & (lastDir.ordinal() == 0 | lastDir.ordinal() == 3))) ? startNode : null));
            }
            lastDir = next(pointer, lastDir);
            pointer = move(lastDir, pointer);
        }
        //nový/již známý uzel
        Node endNode;
        Node returnNode;
        if (c.hasNode(pointer)) {
            endNode = c.getNode(pointer);
            returnNode = null;
        } else {
            endNode = new Node(pointer, (GFNode) cg.get(pointer));
            c.nodes.add(endNode);
            returnNode = endNode;
        }

        Iterator<Component> it = components.iterator();
        Component comp;
        while (it.hasNext()) {
            comp = it.next();
            if (comp.isSource) {
                if (comp.plusNode == null) {
                    comp.plusNode = endNode;
                }
            }
        }
        c.branches.add(new Branch(startNode, startDir, endNode, lastDir.invert(), components, positions));
        //nastaví směr proudu
        ((GFNode) cg.get(startNode.pos)).setCurrent(startDir, -1);
        ((GFNode) cg.get(endNode.pos)).setCurrent(lastDir.invert(), 1);
        return returnNode;
    }

    private GFNode getNode(Node node) {
        return (GFNode) cg.get(node.pos.row,node.pos.col);
    }

    /**
     * @return vstupní pozice posunutá o políčko daným směrem
     */
    private Position move(Direction dir, Position pointer) {
        switch (dir) {
            case NORTH:
                pointer.row--;
                break;
            case EAST:
                pointer.col++;
                break;
            case SOUTH:
                pointer.row++;
                break;
            case WEST:
                pointer.col--;
        }
        return pointer;
    }

    /**
     * @param pos pozice aktuálního pole
     * @param last výstupní směr z předchozího pole
     * @return výstupní směr z aktuáního pole
     */
    private Direction next(Position pos, Direction last) {
        GridField gf = cg.get(pos);
        if (gf.getClass() == GFWire.class) {
            GFWire wire = (GFWire) gf;
            switch (last) {
                case NORTH:
                    return wire.otherDir(Direction.SOUTH);
                case EAST:
                    return wire.otherDir(Direction.WEST);
                case SOUTH:
                    return wire.otherDir(Direction.NORTH);
                case WEST:
                    return wire.otherDir(Direction.EAST);
            }
        }
        return last;
    }
}
