package ApplicationLayer;

import ApplicationLayer.Circuit.Branch;
import ApplicationLayer.Circuit.Circuit;
import ApplicationLayer.Circuit.Loop;
import ApplicationLayer.Circuit.Node;
import java.util.ArrayList;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class Equation {

    //rovnice má tvar: k1*I1+k2*I2+...+kn*In = value
    public double[] coefficients;//k1...kn
    public double value;//1.KZ: 0 2.KZ: algebraický součet napětí ve smyčce

    /**
     * vytvoří rovnici podle 1.KZ
     */
    public Equation(Node node, Circuit c) {
        int nVariables = c.branches.size();
        ArrayList<Branch> branches = c.branches(node, false);
        coefficients = new double[nVariables];
        Branch br;
        for (int i = 0; i < nVariables; i++) {
            br = c.branches.get(i);
            if (branches.contains(br)) {
                coefficients[i] = ((br.nodes.get(0) == node & br.current == -1) | ((br.nodes.get(1) == node & br.current == 1))) ? 1 : -1;
            } else {
                coefficients[i] = 0;
            }
        }
        value = 0;
    }

    /**
     * vytvoří rovnici podle 2.KZ
     */
    public Equation(Loop loop, Circuit c) throws ShortCircuitException {
        int nVariables = c.branches.size();
        coefficients = new double[nVariables];
        Branch br;
        value = 0;
        boolean shortCircuit = true;
        for (int i = 0; i < nVariables; i++) {
            br = c.branches.get(i);
            if (loop.branches.contains(br)) {
                coefficients[i] = (loop.nodes.get(loop.branches.indexOf(br)) == br.getFirstNode()) ? br.getResistance() : -br.getResistance();
                if (coefficients[i] != 0) {
                    shortCircuit = false;//pokud má alespoň 1 větev odpor, není smyčka zkratovaná
                }
                value += (loop.nodes.get(loop.branches.indexOf(br)) == br.getFirstNode()) ? br.getVoltage() : -br.getVoltage();

            } else {
                coefficients[i] = 0;
            }
        }
        value = NumOp.roundDouble(value);
        if (shortCircuit) throw new ShortCircuitException();
    }

    @Override
    public String toString() {
        String s = "";
        double coef;
        boolean first = true;
        coef = coefficients[0];
        for (int i = 0; i < coefficients.length; i++) {
            coef = coefficients[i];
            if (coef != 0) {
                if (coef == 1) {
                    s = (first) ? s.concat("<i>I</i><sub>" + (i + 1) + "</sub>") : s.concat(" + <i>I</i><sub>" + (i + 1) + "</sub>");
                } else if (coef == -1) {
                    s = s.concat(" - <i>I</i><sub>" + (i + 1) + "</sub>");
                } else if (coef > 0) {
                    s = (first) ? s.concat(NumOp.writeDouble(coef) + "<i>I</i><sub>" + (i + 1) + "</sub>") : s.concat(" + " + NumOp.writeDouble(coef) + "<i>I</i><sub>" + (i + 1) + "</sub>");
                } else if (coef < 0) {
                    s = (first) ? s.concat("- " + NumOp.writeDouble(-coef) + "<i>I</i><sub>" + (i + 1) + "</sub>") : s.concat(" - " + NumOp.writeDouble(-coef) + "<i>I</i><sub>" + (i + 1) + "</sub>");
                }
                first = false;
            }
        }
        s = s.concat("</i> = " + NumOp.writeDouble(value));
        //System.out.println(s);
        return s;
    }

    /*Rozpoznávání lineární ne/závislosti rovnic
    public boolean isIn(ArrayList<Equation> equations) {
        Iterator<Equation> it = equations.iterator();
        //System.out.println("Is In " + this);
        while (it.hasNext()) {
            if (is(it.next())) {
                //System.out.println("This is same");
                return true;
            }
        }
        return false;
    }

    private boolean is(Equation e) {
        double coef;
        //System.out.println("Eq ? " + e);
        if (value == 0) {
            if (e.value != 0) {
                return false;
            } else {
                coef = e.coefficients[0] / coefficients[0];
            }
        } else {
            coef = e.value / value;
        }
        for (int i = 0; i < coefficients.length; i++) {
            if (coef * coefficients[i] != e.coefficients[i]) {
                return false;
            }
        }
        return true;
    }*/

    /**
     * @return "velikost" rovnice (počet nenulových koeficientů)
     */
    public int getSize() {
        int size = 0;
        for (int i = 0; i < coefficients.length; i++) {
            if (coefficients[i] != 0) {
                size++;
            }
        }
        return size;
    }
}
