package ApplicationLayer;

import ApplicationLayer.Circuit.Circuit;
import ApplicationLayer.Circuit.Loop;
import ApplicationLayer.Circuit.Node;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.commons.math.linear.Array2DRowRealMatrix;
import org.apache.commons.math.linear.ArrayRealVector;
import org.apache.commons.math.linear.DecompositionSolver;
import org.apache.commons.math.linear.LUDecompositionImpl;
import org.apache.commons.math.linear.RealMatrix;
import org.apache.commons.math.linear.RealVector;
import org.apache.commons.math.linear.SingularMatrixException;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class Equations {

    public static Equations self = new Equations();
    public ArrayList<Equation> equations = new ArrayList<Equation>();//soustava rovnic
    private double[][] coefficients;//matice koeficientů
    private double[] values;//pole hodnot na pravé straně rovnic
    private double[] solution;//pole řešení
    private int firstTypeEqN;

    public void generateEquations(Circuit c) throws ShortCircuitException {
        equations.clear();
        firstTypeEqN = c.firsLawEquationsNeeded();
        int eqs = c.firsLawEquationsNeeded() + c.secondLawEquationsNeeded();
        coefficients = new double[eqs][c.branches.size()];
        values = new double[eqs];
        //Rce 1.KZ
        Iterator<Node> itN = c.nodes.iterator();
        Node node;
        while (itN.hasNext()) {
            node = itN.next();
            //System.out.println("\nEq Node ");
            if (equations.size() < c.firsLawEquationsNeeded()) {
                equations.add(new Equation(node, c));
                coefficients[equations.size() - 1] = equations.get(equations.size() - 1).coefficients;
                values[equations.size() - 1] = equations.get(equations.size() - 1).value;
                //System.out.println("used");
            }
        }
        //Rce 2.KZ
        Iterator<Loop> itL = c.loops.iterator();
        Loop loop;
        Equation eq;
        while (itL.hasNext()) {
            loop = itL.next();
            eq = new Equation(loop, c);
            equations.add(eq);
            //System.out.println("\nEq Loop " + eq);
            coefficients[equations.size() - 1] = equations.get(equations.size() - 1).coefficients;
            values[equations.size() - 1] = equations.get(equations.size() - 1).value;
            //System.out.println("used");
        }
    }

    /**
     * vyřeší rovnice
     */
    public void count(Circuit c) throws SingularMatrixException{
        RealMatrix matrix = new Array2DRowRealMatrix(coefficients);
        //System.out.println(matrix);
        DecompositionSolver solver = new LUDecompositionImpl(matrix).getSolver();
        RealVector constants = new ArrayRealVector(values);
        RealVector solutions = solver.solve(constants);
        solution = solutions.getData();
        for (int i = 0;i<solution.length;i++){
            solution[i] = NumOp.roundDouble(solution[i]);
        }
    }

    /**
     * uloží řešení, aby bylo dostupné v popisech prvků obvodu
     */
    public void saveSolution(Circuit c) {
        for (int i = 0; i < solution.length; i++) {
            //System.out.println("I" + (i + 1) + " = " + solution[i] + " = " + roundDouble(solution[i], ACCURACY_I));
            c.branches.get(i).setI(NumOp.roundDouble(solution[i]));
        }
    }

    /**
     * @return vrací "velikost" (počet nenulových koeficientů) soustavy rovnic pro daný obvod (a daný systém smyček)
     */
    public static int getPredictedSize(Circuit c) throws ShortCircuitException {
        int size = 0;
        Iterator<Loop> itL = c.loops.iterator();
        Loop loop;
        Equation eq;
        while (itL.hasNext()) {
            loop = itL.next();
            eq = new Equation(loop, c);
            size += eq.getSize();
        }
        return size;
    }

    public double[] getSolution() {
        return solution;
    }
}
