package PresentationLayer.Forms;

import ApplicationLayer.NumOp;
import PresentationLayer.Grid.CircuitGrid;
import PresentationLayer.Grid.GFEmpty;
import PresentationLayer.Grid.GFNode;
import PresentationLayer.Grid.GFNone;
import PresentationLayer.Grid.GFResistor;
import PresentationLayer.Grid.GFSource;
import PresentationLayer.Grid.GFWire;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * umožňuje editovat komponenty
 * @author Jakub Hrncir (2010)
 */
public class ComponentPanel extends JPanel {

    private MainForm mf;
    private CircuitGrid cg = CircuitGrid.self;
    private JLabel lblComponent;
    private JLabel lblParameter;
    public JTextField tfValue;
    private ImageIcon confirmII = createImageIcon("confirm.jpg");
    private JButton btnConfirm;
    private ImageIcon remII = createImageIcon("remove.jpg");
    private JButton btnRemove;
    private ImageIcon sourceII = createImageIcon("newSource.jpg");
    private JButton btnChangePolarity;
    private ImageIcon changePolarityII = createImageIcon("changePolarity.jpg");
    private JButton btnSource;
    private ImageIcon resistorII = createImageIcon("newResistor.jpg");
    private JButton btnResistor;
    private Font textEnabled = MainForm.FONT;
    private Font textDisabled = MainForm.FONT_I;

    public ComponentPanel(MainForm mf) {
        super();
        setPreferredSize(new Dimension(128, 248));
        setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Úprava komponent", TitledBorder.LEFT, TitledBorder.DEFAULT_POSITION, MainForm.FONT_B));
        this.mf = mf;
        init();
    }

    private void init() {
        this.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        lblComponent = new JLabel(" ");
        c.insets = new Insets(-2, 7, 2, 0);
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.NORTHWEST;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        add(lblComponent, c);

        lblParameter = new JLabel("Odpor/Napětí (Ω/V)");
        c.insets = new Insets(0, 7, 0, 0);
        c.anchor = GridBagConstraints.WEST;
        c.gridx = 0;
        c.gridy = 1;
        add(lblParameter, c);

        tfValue = new JTextField("1");
        tfValue.setHorizontalAlignment(JTextField.RIGHT);
        tfValue.setPreferredSize(new Dimension(65, 20));
        c.insets = new Insets(2, 8, 2, 0);
        c.gridx = 0;
        c.gridy = 3;
        c.weightx = 0;
        c.weighty = 1;
        add(tfValue, c);

        btnConfirm = new JButton(confirmII);
        btnConfirm.setPreferredSize(new Dimension(30, 30));
        btnConfirm.setToolTipText("Potvrdit změnu hodnoty (Enter)");
        c.insets = new Insets(2, 8, 2, 8);
        c.fill = GridBagConstraints.NONE;
        c.ipadx = 0;
        c.anchor = GridBagConstraints.EAST;
        c.gridx = 1;
        c.gridy = 3;
        c.weightx = 1;
        c.weighty = 1;
        add(btnConfirm, c);

        btnResistor = new JButton(resistorII);
        btnResistor.setPreferredSize(new Dimension(100, 50));
        btnResistor.setToolTipText("Přidat rezistor");
        c.anchor = GridBagConstraints.CENTER;
        c.gridx = 0;
        c.gridy = 4;
        add(btnResistor, c);

        btnSource = new JButton(sourceII);
        btnSource.setPreferredSize(new Dimension(100, 50));
        btnSource.setToolTipText("Přidat zdroj");
        c.gridx = 0;
        c.gridy = 5;
        add(btnSource, c);

        btnChangePolarity = new JButton(changePolarityII);
        btnChangePolarity.setPreferredSize(new Dimension(40, 40));
        btnChangePolarity.setToolTipText("Změnit polaritu zdroje");
        c.insets = new Insets(2, 8, 2, 8);
        c.anchor = GridBagConstraints.WEST;
        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth = 1;
        add(btnChangePolarity, c);

        btnRemove = new JButton(remII);
        btnRemove.setPreferredSize(new Dimension(40, 40));
        btnRemove.setToolTipText("Odebrat komponentu");
        c.anchor = GridBagConstraints.EAST;
        c.gridx = 1;
        c.gridy = 6;
        add(btnRemove, c);

        disableAll();

        tfValue.addKeyListener(new KeyAdapter() {

            @Override
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    if (btnConfirm.isEnabled()) {
                        btnConfirm.doClick();
                    } else {
                        btnResistor.doClick();
                    }
                }
            }

            @Override
            public void keyTyped(KeyEvent e) {
                super.keyTyped(e);
                //System.out.println("Key typed: " + e.getKeyChar());
                //System.out.println("SelSt:" + tfValue.getSelectionStart());
                //System.out.println("SelEnd:" + tfValue.getSelectionEnd());
                char c = e.getKeyChar();
                if (c == ',' | c == '1' | c == '2' | c == '3' | c == '4' | c == '5' | c == '6' | c == '7' | c == '8' | c == '9' | c == '0') {
                    int cursor = tfValue.getCaretPosition();
                    String number = tfValue.getText();
                    int backup = number.length();
                    if (tfValue.getSelectedText() != null) {
                        number = number.substring(0, tfValue.getSelectionStart()) + c + number.substring(tfValue.getSelectionEnd());
                        if (cursor < tfValue.getSelectionEnd()) {
                            backup -= tfValue.getSelectedText().length();
                        }
                    } else {
                        number = number.substring(0, cursor) + c + number.substring(cursor);
                    }
                    if (isValidN(number)) {
                        changeTFValue(number);
                    }
                    //System.out.println("Backup: " + backup + "  Length: " + tfValue.getText().length() + "  Caret: " + cursor);
                    tfValue.setCaretPosition(cursor + tfValue.getText().length() - backup);
                }
                e.consume();
            }
        });
        btnResistor.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                setN();
                double val = NumOp.parseDouble(tfValue.getText());
                cg.setResistor(val);
                mf.repaintGrid();
                mf.updateP();
            }
        });

        btnSource.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                setN();
                double val = NumOp.parseDouble(tfValue.getText());
                cg.setSource(val);
                mf.repaintGrid();
                mf.updateP();
            }
        });

        btnRemove.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cg.removeComponent();
                mf.repaintGrid();
                mf.updateP();
            }
        });

        btnConfirm.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                setN();
                double val = NumOp.parseDouble(tfValue.getText());
                cg.changeComponentValue(val);
                mf.repaintGrid();
            }
        });

        btnChangePolarity.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cg.changeSourcePolarity();
                mf.repaintGrid();
            }
        });
    }

    /**
     * aktualizuje komponenty
     */
    public void updatePannel() {
        if (cg.getSelected().getClass().equals(GFNone.class) | cg.getSelected().getClass().equals(GFEmpty.class) | cg.getSelected().getClass().equals(GFNode.class)) {
            disableAll();
            return;
        }
        if (cg.getSelected().getClass().equals(GFResistor.class)) {
            disableAll();
            enableComponentEdit();
            btnChangePolarity.setEnabled(false);
            lblComponent.setText("Rezistor");
            lblParameter.setText("Odpor (Ω):");
            tfValue.setText(NumOp.writeDouble(((GFResistor) cg.getSelected()).getResistance()));
            return;
        }
        if (cg.getSelected().getClass().equals(GFSource.class)) {
            disableAll();
            enableComponentEdit();
            btnChangePolarity.setEnabled(true);
            lblComponent.setText("Zdroj");
            lblParameter.setText("Napětí (V):");
            tfValue.setText(NumOp.writeDouble(((GFSource) cg.getSelected()).getVoltage()));
            return;
        }
        if (cg.getSelected().getClass().equals(GFWire.class)) {
            disableAll();
            if (((GFWire) cg.getSelected()).canPutComponent()) {
                lblComponent.setText("Zadejte hodnotu:");
                lblComponent.setFont(textEnabled);
                lblComponent.setForeground(Color.BLACK);
                btnResistor.setEnabled(true);
                btnSource.setEnabled(true);
                tfValue.setEnabled(true);
                lblParameter.setFont(textEnabled);
                lblParameter.setForeground(Color.BLACK);
                lblParameter.setText("Odpor/Napětí (Ω/V)");
            }
            return;
        }
    }

    /**
     * deaktivuje editační komponenty
     */
    private void disableComponentEdit() {
        lblComponent.setText(" ");
        lblComponent.setFont(textDisabled);
        lblComponent.setForeground(Color.GRAY);
        lblParameter.setFont(textDisabled);
        lblParameter.setForeground(Color.GRAY);
        tfValue.setEnabled(false);
        btnConfirm.setEnabled(false);
        btnRemove.setEnabled(false);
        btnChangePolarity.setEnabled(false);
    }

    /**
     * zaktivuje editační komponenty
     */
    private void enableComponentEdit() {
        lblComponent.setFont(textEnabled);
        lblComponent.setForeground(Color.BLACK);
        lblParameter.setFont(textEnabled);
        lblParameter.setForeground(Color.BLACK);
        tfValue.setEnabled(true);
        btnConfirm.setEnabled(true);
        btnRemove.setEnabled(true);
    }

    /**
     * deaktivuje celý panel
     */
    private void disableAll() {
        disableComponentEdit();
        btnResistor.setEnabled(false);
        btnSource.setEnabled(false);
    }

    private static ImageIcon createImageIcon(String path) {
        return MainForm.createImageIcon(path);

    }

    /**
     * @return hodnota v text fieldu
     */
    public Double getComponentValue() {
        setN();
        return NumOp.parseDouble(tfValue.getText());
    }

    /**
     * testuje:
     * pouze 1 desetninná čárka
     * max 7 číslic celkem
     * max 3 číslice za desetinnou čárkou
     * @param number
     * @return odpovídá formát čísla?
     */
    private boolean isValidN(String number) {
        int comma = number.indexOf(",");
        if (comma != number.lastIndexOf(",")) {
            return false;
        }
        if (comma > -1) {
            if (comma == 7 | number.length() > 8) {
                return false;
            }
            if (number.substring(comma + 1).length() > 3) {
                return false;
            }
        } else {
            if (number.length() > 7) {
                return false;
            }
        }
        return true;
    }

    /**
     * zapíše number do tfValue
     * pokud number začíná čárkou, doplní nulu
     * pokud obsahuje zbytečné nuly na začátku, odstraní je
     * @param number
     */
    private void changeTFValue(String number) {
        if (number.charAt(0) == ',') {
            number = "0" + number;
        }
        while (number.startsWith("00")) {
            number = number.substring(1);
        }
        tfValue.setText(number);
    }

    /**
     * zkontroluje validitu čísla v tfValue, nevhodné hodnoty případně nahradí 1
     * poslední kontrola před využitím čísla při změně nebo přidání komponenty
     */
    private void setN() {
        String number = tfValue.getText();
        if (isValidN(number)) {
            changeTFValue(number);
            double val = NumOp.parseDouble(tfValue.getText());
            if (val == 0) {
                tfValue.setText("1");
            }
        } else {
            tfValue.setText("1");
        }
    }
}
