package PresentationLayer.Forms;

import ApplicationLayer.ApplicationLayer;
import ApplicationLayer.Circuit.*;
import ApplicationLayer.Equation;
import PresentationLayer.Grid.CircuitGrid;
import PresentationLayer.Grid.GFNode;
import PresentationLayer.Grid.GFWire;
import PresentationLayer.GridPannel;
import PresentationLayer.IPresentationLayer;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.html.HTMLEditorKit;

/**
 * Hlavní formulář aplikace
 * @author Jakub Hrncir (2010)
 */
public class MainForm extends JFrame implements IPresentationLayer {

    private JPanel mainPanel;
    private JPanel toolPanel;
    private JPanel topPanel;
    private ResizePanel resizePanel;
    private ComponentPanel componentPanel;
    private CircuitPanel circuitPanel;
    private JButton btnNewGrid;
    private ImageIcon newGridII;
    private JButton btnZoomIn;
    private ImageIcon zoomInII;
    private JButton btnZoomOut;
    private ImageIcon zoomOutII;
    private JButton btnNewNode;
    private ImageIcon newNodeII;
    private JButton btnRemNode;
    private ImageIcon remNodeII;
    private JButton btnProcess;
    private ImageIcon processII;
    private JButton btnReturn;
    private JButton btnAbout;
    private ImageIcon returnII;
    private GridPannel gridPanel;
    private JScrollPane scrollPane;
    private JScrollPane scrollPaneMessages;
    private JEditorPane edPaneMessages;
    private JScrollPane scrollPaneEqs;
    private JEditorPane edPaneEqs;
    private JScrollPane scrollPaneSol;
    private JEditorPane edPaneSol;
    private String navod_obvod;
    private String navod_rce;
    private String header;
    private JTabbedPane tabbedPane;
    private MFState edPaneMessagesState;
    private JScrollPane scrollPaneList;
    private JSplitPane splitPane;
    private JList list;
    private JPanel listPanel;
    private DefaultListModel listModel;
    private JFrame frame;
    private int x;
    private int y;
    private CircuitGrid cg = CircuitGrid.self;
    private MFState state;
    private Circuit circuit;
    private boolean needInterpret = false;
    public static final String FONT_NAME = "Arial";
    public static final Font FONT = new Font(FONT_NAME, Font.PLAIN, 12);
    public static final Font FONT_B = new Font(FONT_NAME, Font.BOLD, 12);
    public static final Font FONT_I = new Font(FONT_NAME, Font.ITALIC, 12);

    public MainForm() {
        super("Kirchhoff");
        setSize(1024, 700);
        setResizable(true);
        setMinimumSize(new Dimension(800, 600));
        setLocation(0, 0);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        state = MFState.EDITING;
        init();
        addComponentsToPane(getContentPane());
        setVisible(true);
    }

    private void init() {
        //ikony tlačítek
        newGridII = createImageIcon("newGrid.jpg");
        zoomInII = createImageIcon("zoomIn.jpg");
        zoomOutII = createImageIcon("zoomOut.jpg");
        newNodeII = createImageIcon("newNode.jpg");
        remNodeII = createImageIcon("remNode.jpg");
        processII = createImageIcon("process.jpg");
        returnII = createImageIcon("return.jpg");
        //html texty
        navod_obvod = ApplicationLayer.loadFileToString("html/navod_obvod.html");
        navod_rce = ApplicationLayer.loadFileToString("html/navod_rce.html");
        header = ApplicationLayer.loadFileToString("html/header.html");
    }

    private void addComponentsToPane(Container pane) {
        //frame pro dialogy a zprávy
        frame = new JFrame();

        //základní layout
        pane.setLayout(new BorderLayout());

        //část s obvodem
        gridPanel = new GridPannel(this);
        scrollPane = new JScrollPane(gridPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

        //nástroje a informace vpravo – toolPanel
        toolPanel = new JPanel(new GridBagLayout());
        toolPanel.setMinimumSize(new Dimension(128, 450));
        GridBagConstraints c = new GridBagConstraints();

        //prvky toolPanelu
        //resizePanel - změna velikosti gridu
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 0;
        resizePanel = new ResizePanel(this);
        toolPanel.add(resizePanel, c);

        //componentPanel - úprava komponent
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 0;
        c.weighty = 1;
        componentPanel = new ComponentPanel(this);
        componentPanel.setVisible(true);
        toolPanel.add(componentPanel, c);

        //circuitPanel - vlastnosti prvků obvodu
        c.gridx = 0;
        c.gridy = 2;
        c.weightx = 0;
        c.weighty = 1;
        c.fill = GridBagConstraints.BOTH;
        circuitPanel = new CircuitPanel(this);
        circuitPanel.setVisible(false);
        toolPanel.add(circuitPanel, c);

        //list částí obvodu
        listPanel = new JPanel(new BorderLayout());
        listPanel.setPreferredSize(new Dimension(260, 120));
        listPanel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Seznam prvků obvodu", TitledBorder.LEFT, TitledBorder.DEFAULT_POSITION, FONT_B));
        listPanel.setVisible(false);
        listModel = new DefaultListModel();
        list = new JList(listModel);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setFixedCellHeight(20);
        list.setFont(new Font(FONT_NAME, Font.PLAIN, 12));
        scrollPaneList = new JScrollPane(list);
        scrollPaneList.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        listPanel.add(scrollPaneList);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 3;
        c.weightx = 0;
        c.weighty = 1;
        toolPanel.add(listPanel, c);

        //výpisy
        edPaneMessages = new JEditorPane();
        edPaneMessages.setEditable(false);
        edPaneMessages.setEditorKit(new HTMLEditorKit());
        edPaneMessagesState = MFState.EDITING;
        edPaneMessages.setText(navod_obvod);
        scrollPaneMessages = new JScrollPane(edPaneMessages);
        scrollPaneMessages.setPreferredSize(new Dimension(520, 200));
        scrollPaneMessages.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        edPaneEqs = new JEditorPane();
        edPaneEqs.setEditable(false);
        edPaneEqs.setEditable(false);
        edPaneEqs.setEditorKit(new HTMLEditorKit());
        edPaneMessagesState = MFState.EDITING;
        scrollPaneEqs = new JScrollPane(edPaneEqs);
        scrollPaneEqs.setPreferredSize(new Dimension(520, 200));
        scrollPaneEqs.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        edPaneSol = new JEditorPane();
        edPaneSol.setEditable(false);
        edPaneSol.setEditable(false);
        edPaneSol.setEditorKit(new HTMLEditorKit());
        edPaneMessagesState = MFState.EDITING;
        scrollPaneSol = new JScrollPane(edPaneSol);
        scrollPaneSol.setPreferredSize(new Dimension(520, 200));
        scrollPaneSol.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        tabbedPane = new JTabbedPane(JTabbedPane.TOP);
        tabbedPane.addTab("Návod", scrollPaneMessages);

        //nástroje nahoře topPanel
        topPanel = new JPanel(new GridBagLayout());
        topPanel.setPreferredSize(new Dimension(450, 30));

        //prvky topPanelu
        btnNewGrid = new JButton(newGridII);
        btnNewGrid.setToolTipText("Nový obvod");
        btnNewGrid.setPreferredSize(new Dimension(30, 30));
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        c.weighty = 0;
        topPanel.add(btnNewGrid, c);

        btnZoomIn = new JButton(zoomInII);
        btnZoomIn.setToolTipText("Přiblížit");
        btnZoomIn.setPreferredSize(new Dimension(30, 30));
        c.gridx = 1;
        topPanel.add(btnZoomIn, c);

        btnZoomOut = new JButton(zoomOutII);
        btnZoomOut.setToolTipText("Oddálit");
        btnZoomOut.setPreferredSize(new Dimension(30, 30));
        c.gridx = 2;
        topPanel.add(btnZoomOut, c);

        btnNewNode = new JButton(newNodeII);
        btnNewNode.setEnabled(false);
        btnNewNode.setToolTipText("Přidat uzel");
        btnNewNode.setPreferredSize(new Dimension(30, 30));
        c.gridx = 3;
        topPanel.add(btnNewNode, c);

        btnRemNode = new JButton(remNodeII);
        btnRemNode.setEnabled(false);
        btnRemNode.setToolTipText("Odstranit uzel");
        btnRemNode.setPreferredSize(new Dimension(30, 30));
        c.gridx = 4;
        topPanel.add(btnRemNode, c);

        btnProcess = new JButton("Zpracovat obvod", processII);
        btnProcess.setEnabled(true);
        btnProcess.setFont(FONT);
        btnProcess.setToolTipText("Pokud máte obvod hotový, klepněte sem");
        btnProcess.setHorizontalTextPosition(SwingConstants.RIGHT);
        btnProcess.setPreferredSize(new Dimension(140, 30));
        btnProcess.setMargin(new Insets(0, -17, 0, 0));
        btnProcess.setFocusPainted(false);
        c.insets = new Insets(0, 0, 0, 0);
        c.gridx = 5;
        topPanel.add(btnProcess, c);

        btnReturn = new JButton("Změnit obvod", returnII);
        btnReturn.setEnabled(false);
        btnReturn.setFont(FONT);
        btnReturn.setHorizontalTextPosition(SwingConstants.RIGHT);
        btnReturn.setPreferredSize(new Dimension(115, 30));
        btnReturn.setMargin(new Insets(0, -6, 0, 0));
        btnReturn.setFocusPainted(false);
        c.gridx = 6;
        topPanel.add(btnReturn, c);

        btnAbout = new JButton("O programu");
        btnAbout.setFont(FONT);
        btnAbout.setFocusPainted(false);
        c.anchor = GridBagConstraints.EAST;
        c.gridx = 7;
        c.weightx = 1;
        topPanel.add(btnAbout, c);


        //umístění základních komponent
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.setPreferredSize(new Dimension(660, 400));
        mainPanel.add(scrollPane, BorderLayout.CENTER);
        mainPanel.add(toolPanel, BorderLayout.EAST);
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, mainPanel, tabbedPane);
        splitPane.getLeftComponent().setMinimumSize(new Dimension(460, 400));
        splitPane.getRightComponent().setMinimumSize(new Dimension(200, 400));
        splitPane.setOneTouchExpandable(true);
        splitPane.setResizeWeight(1);
        splitPane.setDividerLocation(660);
        pane.add(splitPane, BorderLayout.CENTER);
        pane.add(topPanel, BorderLayout.NORTH);

        //Listenery
        addComponentListener(new ComponentListener() {

            public void componentResized(ComponentEvent e) {
                repaintWholeGrid();
            }

            public void componentMoved(ComponentEvent e) {
            }

            public void componentShown(ComponentEvent e) {
            }

            public void componentHidden(ComponentEvent e) {
            }
        });
        gridPanel.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
                if (state == MFState.EDITING) {
                    gridPanel.mouseClick(e.getX(), e.getY(), e.getButton());
                    repaintGrid();
                    updateP();
                }
            }

            public void mousePressed(MouseEvent e) {
                x = e.getX();
                y = e.getY();
            }

            public void mouseReleased(MouseEvent e) {
                if (state == MFState.EDITING) {
                    if ((x != e.getX()) | (y != e.getY())) {
                        gridPanel.mouseDrag(x, y, e.getX(), e.getY(), e.getButton(), state);
                        repaintGrid();
                        updateP();
                    }
                }
            }

            public void mouseEntered(MouseEvent e) {
            }

            public void mouseExited(MouseEvent e) {
            }
        });
        gridPanel.addMouseMotionListener(new MouseMotionListener() {

            public void mouseDragged(MouseEvent e) {
                if (state.ordinal() > 0) {
                    return;
                }
                if (gridPanel.mouseDraging(x, y, e.getX(), e.getY())) {
                    repaintGrid();
                }
            }

            public void mouseMoved(MouseEvent e) {
            }
        });
        btnNewGrid.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Object[] options = {"OK", "Storno"};
                int response = JOptionPane.showOptionDialog(frame, "Nový obvod přemaže ten stávající. Pokračovat?", "Nový obvod", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, options, null);
                if (response == JOptionPane.OK_OPTION) {
                    CircuitGrid.self.resetGrid(6, 6);
                    circuitPanel.setVisible(false);
                    listPanel.setVisible(false);
                    componentPanel.setVisible(true);
                    resizePanel.setVisible(true);
                    state = MFState.EDITING;
                    repaintWholeGrid();
                    updateP();
                    listModel.clear();
                }
            }
        });
        btnZoomIn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                btnZoomIn.setEnabled(gridPanel.zoomIn());
                btnZoomOut.setEnabled(true);
                repaintWholeGrid();
            }
        });
        btnZoomOut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                btnZoomOut.setEnabled(gridPanel.zoomOut());
                btnZoomIn.setEnabled(true);
                repaintWholeGrid();
            }
        });
        btnNewNode.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cg.setNode();
                repaintGrid();
                updateP();

            }
        });
        btnRemNode.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cg.removeNode();
                repaintGrid();
                updateP();
            }
        });
        btnProcess.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (state == MFState.EDITING) {
                    try {
                        cg.canProcess();
                        cg.crop();
                        ApplicationLayer.self.processCG();
                        repaintGrid();
                    } catch (CircuitNotClosedException ex) {
                        JOptionPane.showMessageDialog(frame, "Obvod není uzavřený!\nOdstraňte slepé vodiče nebo doplňte ty chybějící.", "Error", JOptionPane.ERROR_MESSAGE);
                    } catch (NoCircuitException ex) {
                        JOptionPane.showMessageDialog(frame, "Žádný nebo nerozvětvený obvod!", "Error", JOptionPane.ERROR_MESSAGE);
                    } catch (NoSourceException ex) {
                        JOptionPane.showMessageDialog(frame, "Obvod nemá zdroj!", "Error", JOptionPane.ERROR_MESSAGE);
                    } catch (NoResistorException ex) {
                        JOptionPane.showMessageDialog(frame, "Obvod nemá žádný odpor!", "Error", JOptionPane.ERROR_MESSAGE);
                    }
                } else if (state == MFState.FORMING_EQUATIONS) {
                    ApplicationLayer.self.generateEquations();
                } else if (state == MFState.EQUATIONS) {
                    ApplicationLayer.self.solveEquations();
                } else if (state == MFState.SOLVED & needInterpret) {
                    ApplicationLayer.self.interpretSolution();
                }

            }
        });
        btnReturn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                state = MFState.EDITING;
                cg.deselect();
                cg.unHighlight();
                updateP();
                repaintGrid();
            }
        });
        btnAbout.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(frame, new InfoPanel(), "O programu Kirchhoff", JOptionPane.PLAIN_MESSAGE);
            }
        });
        list.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent e) {
                if (list.getSelectedIndex() != -1) {
                    cg.highlight((Highlightable) listModel.getElementAt(list.getSelectedIndex()), gridPanel.getGraphics());
                    circuitPanel.setText(circuit.getPartDescription(state, list.getSelectedValue()));
                } else {
                    circuitPanel.setText("");
                }
            }
        });
        edPaneMessages.addHyperlinkListener(new HyperlinkListener() {

            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    Desktop d = Desktop.getDesktop();
                    try {
                        d.browse(e.getURL().toURI());
                    } catch (IOException ex) {
                    } catch (URISyntaxException ex) {
                    }
                }
            }
        });
    }

    /**
     * @return hodnota editu na component pannelu
     */
    public double getComponentValue() {
        return componentPanel.getComponentValue();
    }

    /**
     * překreslí Grid
     * ! velikost musí být zachována
     */
    public void repaintGrid() {
        gridPanel.paintComponent(gridPanel.getGraphics());
    }

    /**
     * překreslí Grid a přizpůsobí scrollbary
     */
    public void repaintWholeGrid() {
        gridPanel.paintWholeComponent(gridPanel.getGraphics(), scrollPane.getWidth(), scrollPane.getHeight());
        scrollPane.setViewportView(gridPanel);
    }

    /**
     * načte ikonu ze souboru
     * @param path cesta k obrázku
     * @return ImageIcon
     */
    public static ImageIcon createImageIcon(String path) {

        try {
            InputStream is = MainForm.class.getResourceAsStream("images/" + path);
            BufferedInputStream bis = new BufferedInputStream(is);
            byte[] byBuf = new byte[10000];
            int byteRead = bis.read(byBuf, 0, 10000);
            ImageIcon icon = new ImageIcon(byBuf);
            return icon;
        } catch (Exception ex) {
        }

        return new ImageIcon("images/" + path);
    }

    /**
     * aktualizuje nástroje
     */
    public void updateP() {
        componentPanel.updatePannel();
        if (state == MFState.EDITING) {
            if (cg.getSelected().getClass().equals(GFWire.class)) {
                btnNewNode.setEnabled(true);
            } else {
                btnNewNode.setEnabled(false);
            }
            if (cg.getSelected().getClass().equals(GFNode.class) && !((GFNode) cg.getSelected()).needNode()) {
                btnRemNode.setEnabled(true);
            } else {
                btnRemNode.setEnabled(false);
            }
            btnProcess.setEnabled(true);
            btnProcess.setText("Zpracovat obvod");
            btnProcess.setToolTipText("Pokud máte obvod hotový, klepněte sem");
            btnProcess.setMargin(new Insets(0, -17, 0, 0));
            btnReturn.setEnabled(false);
            circuitPanel.setVisible(false);
            listPanel.setVisible(false);
            componentPanel.setVisible(true);
            resizePanel.setVisible(true);
            if (!state.equals(edPaneMessagesState)) {
                edPaneMessagesState = MFState.EDITING;
                edPaneMessages.setText(navod_obvod);
            }
            tabbedPane.removeAll();
            tabbedPane.add("Návod", scrollPaneMessages);
        }
        if (state.ordinal() > 0) {
            btnNewNode.setEnabled(false);
            btnRemNode.setEnabled(false);
            btnReturn.setEnabled(true);

        }
        if (state == MFState.FORMING_EQUATIONS) {
            btnProcess.setEnabled(true);
            btnProcess.setText("Ukázat rovnice");
            btnProcess.setToolTipText("Klepnutím sem zobrazíte vytvořené rovnice");
            btnProcess.setMargin(new Insets(0, -27, 0, 0));
            if (!state.equals(edPaneMessagesState)) {
                edPaneMessagesState = MFState.FORMING_EQUATIONS;
                edPaneMessages.setText(navod_rce);
            }
        }
        if (state == MFState.EQUATIONS) {
            btnProcess.setEnabled(true);
            btnProcess.setText("Spočítat rovnice");
            btnProcess.setToolTipText("Klepnutím sem zobrazíte řešení rovnic");
            btnProcess.setMargin(new Insets(0, -19, 0, 0));
        }
        if (state == MFState.SOLVED) {
            if (!needInterpret) {
                btnProcess.setEnabled(false);
            } else {
                btnProcess.setText("Aktualizovat obvod");
                btnProcess.setToolTipText("Klepnutím sem zobrazíte výsledné směry proudu");
                btnProcess.setMargin(new Insets(0, -8, 0, 0));
            }
        }
        if (state == MFState.INTERPRETED) {
            btnProcess.setEnabled(false);
        }
    }

    /**
     * zobrazí prvky obvodu, přejde k vytváření rovnic
     * @param c
     */
    public void showBranches(Circuit c) {
        circuit = c;
        circuitPanel.setVisible(true);
        listPanel.setVisible(true);
        listModel.clear();
        componentPanel.setVisible(false);
        edPaneMessagesState = MFState.FORMING_EQUATIONS;
        edPaneMessages.setText(navod_rce);
        resizePanel.setVisible(false);
        cg.deselect();
        state = MFState.FORMING_EQUATIONS;
        repaintGrid();
        updateP();
        Iterator<Node> itN = c.nodes.iterator();
        while (itN.hasNext()) {
            listModel.addElement(itN.next());
        }
        Iterator<Branch> itB = c.branches.iterator();
        while (itB.hasNext()) {
            listModel.addElement(itB.next());
        }
        Iterator<Loop> itL = c.loops.iterator();
        while (itL.hasNext()) {
            listModel.addElement(itL.next());
        }
        showMessages();
    }

    /**
     * zobrazí vytvořené rovnice
     * @param equations
     */
    public void showEquations(ArrayList<Equation> equations) {
        list.clearSelection();
        cg.unHighlight();
        repaintGrid();
        state = MFState.EQUATIONS;
        tabbedPane.addTab("Rovnice", scrollPaneEqs);
        edPaneEqs.setText(getEquationsText(equations));
        tabbedPane.setSelectedIndex(1);
        updateP();
        showMessages();
    }

    /**
     * zobrazí řešení
     * @param solution
     * @param needInterpret
     */
    public void showSolution(double[] solution, boolean needInterpret) {
        this.needInterpret = needInterpret;
        list.clearSelection();
        cg.unHighlight();
        repaintGrid();
        tabbedPane.addTab("Řešení", scrollPaneSol);
        edPaneSol.setText(getSolutionsText(solution));
        tabbedPane.setSelectedIndex(2);
        state = MFState.SOLVED;
        updateP();
        showMessages();
    }

    /**
     * zajistí viditelnost části s texty (nápověda,rovnice,řešení) a viditelnost mřížky
     */
    private void showMessages() {
        int dl = splitPane.getDividerLocation();
        //System.out.println("Divider: " + dl + " width:" + splitPane.getWidth());
        if (dl > splitPane.getWidth() - 30) {
            splitPane.setDividerLocation(660);
        }
        if (dl < 461) {
            splitPane.setDividerLocation(461);
        }
    }

    /**
     * interpretuje řešení
     */
    public void interpretSolution() {
        state = MFState.INTERPRETED;
        list.clearSelection();
        repaintGrid();
        updateP();
    }

    /**
     * zobrazí chybovou hlášku
     * @param message
     */
    public void showErrorMessage(String message) {
        JOptionPane.showMessageDialog(frame, message, "Error", JOptionPane.ERROR_MESSAGE);
    }

    /**
     * @return je potřeba zobrazovat oznaceni uzlu a vetvi?
     */
    public boolean needMarks() {
        if (state.ordinal() == 0) {
            return false;
        }
        return true;
    }

    /**
     * vygeneruje text pro vytvořené rovnice
     * @param equations
     * @return
     */
    private String getEquationsText(ArrayList<Equation> equations) {
        String s = "";
        int first = circuit.firsLawEquationsNeeded();
        s = s.concat(header);
        s = s.concat("<h2>Vytvořené rovnice</h2><h3>První Kirchhoffův zákon</h3>");
        for (int i = 0; i < first; i++) {
            s = s.concat("<p>" + equations.get(i).toString() + "</p>");
        }
        s = s.concat("<h3>Druhý Kirchhoffův zákon</h3>");
        for (int i = first; i < equations.size(); i++) {
            s = s.concat("<p>" + equations.get(i).toString() + "</p>");
        }
        return s;
    }

    /**
     * vygeneruje text pro vypočtená řešení
     * @param solution
     * @return
     */
    private String getSolutionsText(double[] solution) {
        String s = "";
        int branches = circuit.branches.size();
        boolean zapor = false;
        s = s.concat(header);
        s = s.concat("<h2>Vypočtené hodnoty proudu</h2>");
        for (int i = 0; i < branches; i++) {
            if (solution[i] < 0) {
                zapor = true;
            }
            s = s.concat("<p> <i>I</i><sub>" + (i + 1) + "</sub> = " + String.format("%.4f", solution[i]) + " A</p>");
        }
        if (zapor) {
            s = s.concat("<p>Záporné hodnoty u některých velikostí proudů znamenají, že proud danou větví proudí ve skutečnosti opačným směrem, než jsem zvolili. Jeho směr je tedy opačný, než jak ukazují šipky v nákresu.</p>");
            s = s.concat("<p>Pokud zvolené směry proudu nesouhlasí s výslednými, můžete jejich směry v nákresu změnit klepnutím na tlačítko <strong>Aktualizovat obvod</strong> (budou upraveny i příslušné hodnoty proudů v popisech prvků obvodu).</p>");
        }
        s = s.concat("<p>Pokud je hodnota proudu nula, proud touto větví vůbec neteče, nebo je menší než 0,1 mA.</p>");
        return s;
    }
}
