package PresentationLayer.Forms;

import ApplicationLayer.Direction;
import PresentationLayer.Grid.CircuitGrid;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * Panel umožňující změnu velikosti mřížky.
 * @author Jakub Hrncir (2010)
 */
public class ResizePanel extends JPanel {

    private MainForm mf;
    private JButton addNorth;
    private JButton addEast;
    private JButton addSouth;
    private JButton addWest;
    private JButton cropGrid;
    private ImageIcon anII = createImageIcon("addNorth.jpg");
    private ImageIcon aeII = createImageIcon("addEast.jpg");
    private ImageIcon asII = createImageIcon("addSouth.jpg");
    private ImageIcon awII = createImageIcon("addWest.jpg");
    private ImageIcon crII = createImageIcon("cropGrid.jpg");
    private int extensionPower = 2;

    public ResizePanel(MainForm mf) {
        super();
        setPreferredSize(new Dimension(128, 142));
        setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED),"Velikost mřížky", TitledBorder.LEFT,TitledBorder.DEFAULT_POSITION,MainForm.FONT_B));
        this.mf = mf;
        init();
    }

    private void init() {
        this.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        addNorth = new JButton(anII);
        addNorth.setPreferredSize(new Dimension(36, 36));
        addNorth.setToolTipText("Přidá 2 řádky mřížky nahoře");
        addEast = new JButton(aeII);
        addEast.setPreferredSize(new Dimension(36, 36));
        addEast.setToolTipText("Přidá 2 sloupce mřížky vpravo");
        addSouth = new JButton(asII);
        addSouth.setPreferredSize(new Dimension(36, 36));
        addSouth.setToolTipText("Přidá 2 řádky mřížky dole");
        addWest = new JButton(awII);
        addWest.setPreferredSize(new Dimension(36, 36));
        addWest.setToolTipText("Přidá 2 sloupce mřížky vlevo");
        cropGrid = new JButton(crII);
        cropGrid.setPreferredSize(new Dimension(36, 36));
        cropGrid.setToolTipText("Oříznout mřížku");

        c.anchor = GridBagConstraints.NORTH;
        c.insets = new Insets(1, 1, 1, 1);
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 0;
        c.weighty = 0;
        add(addNorth, c);
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 0;
        c.weighty = 0;
        add(addWest, c);
        c.gridx = 1;
        c.gridy = 1;
        c.weightx = 0;
        c.weighty = 0;
        add(cropGrid, c);
        c.gridx = 2;
        c.gridy = 1;
        c.weightx = 0.1;
        c.weighty = 0;
        add(addEast, c);
        c.gridx = 1;
        c.gridy = 2;
        c.weightx = 0;
        c.weighty = 0.1;
        add(addSouth, c);

        addNorth.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                CircuitGrid.self.extend(Direction.NORTH, extensionPower);
                mf.repaintWholeGrid();
            }
        });
        addEast.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                CircuitGrid.self.extend(Direction.EAST, extensionPower);
                mf.repaintWholeGrid();
            }
        });
        addSouth.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                CircuitGrid.self.extend(Direction.SOUTH, extensionPower);
                mf.repaintWholeGrid();
            }
        });
        addWest.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                CircuitGrid.self.extend(Direction.WEST, extensionPower);
                mf.repaintWholeGrid();
            }
        });
        cropGrid.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                CircuitGrid.self.crop();
                mf.repaintWholeGrid();
                mf.updateP();
            }
        });
    }

    protected static ImageIcon createImageIcon(String path) {
        return MainForm.createImageIcon(path);
    }
}

