package PresentationLayer.Grid;

import ApplicationLayer.Circuit.Highlightable;
import ApplicationLayer.Circuit.Loop;
import ApplicationLayer.Position;
import ApplicationLayer.Direction;
import PresentationLayer.Forms.CircuitNotClosedException;
import PresentationLayer.Forms.NoCircuitException;
import PresentationLayer.Forms.NoResistorException;
import PresentationLayer.Forms.NoSourceException;
import PresentationLayer.GridPannel;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.awt.image.BufferedImage;
import java.util.Iterator;

/**
 * obsahuje data mřížky
 * @author Jakub Hrncir (2010)
 */
public class CircuitGrid {

    public static CircuitGrid self = new CircuitGrid(6, 6);
    private static GFNone none = new GFNone();
    private ArrayList<CircuitGridLine> gridlines;
    private int GRIDCONSTANT = 60;//aktuální velikost políčka
    private int MINGRIDSIZE = 3;//minimální počet sloupců a řádků
    private int MINFIELDSIZE = 40;//minimální velikost políčka
    private int MAXFIELDSIZE = 150;//maximální velikost políčka
    private GridField lastDragSelected = none;
    private GridField selected = none;
    private int selectedX;
    private int selectedY;
    public GridPannel gp;

    private CircuitGrid(int row, int col) {
        gridlines = new ArrayList<CircuitGridLine>();
        selected = none;
        for (int i = 0; i < row; i++) {
            gridlines.add(new CircuitGridLine(col));
        }
    }

    public void resetGrid(int row, int col) {
        gridlines.clear();
        for (int i = 0; i < row; i++) {
            gridlines.add(new CircuitGridLine(col));
        }
        selected = none;
    }

    /**
     * přidá dráty do prázdných políček a do políček s dráty a uzly
     * přidá uzly, pokud je to nutné (tzn 3 větve v poli)
     * @param where řádek/sloupec(rozlišeno par. horizontal), kam se přidává drát
     * @param from políčko, kde drát začíná
     * @param to políčko, kde drát končí TO>FROM
     * @param horizontal drát je vodorovný/svislý
     */
    public void drawWire(int where, int from, int to, boolean horizontal) {
        if (horizontal) {
            GridField gf = get(where, from);
            if (gf.getClass().equals(GFEmpty.class)) {
                set(where, from, new GFWire(false, true, false, false));
            } else if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                ((GFWire) gf).setEast(true);
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(where, from, new GFNode((GFWire) gf));
                }
            }
            for (int i = 1; i < to - from; i++) {
                gf = get(where, from + i);
                if (gf.getClass().equals(GFEmpty.class)) {
                    set(where, from + i, new GFWire(false, true, false, true));
                }
                if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                    ((GFWire) gf).setEast(true);
                    ((GFWire) gf).setWest(true);
                    if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                        set(where, from + i, new GFNode((GFWire) gf));
                    }
                }
            }
            gf = get(where, to);
            if (gf.getClass().equals(GFEmpty.class)) {
                set(where, to, new GFWire(false, false, false, true));
            } else if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                ((GFWire) gf).setWest(true);
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(where, to, new GFNode((GFWire) gf));
                }
            }
        } else {
            GridField gf = get(from, where);
            if (gf.getClass().equals(GFEmpty.class)) {
                set(from, where, new GFWire(false, false, true, false));
            } else if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                ((GFWire) gf).setSouth(true);
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(from, where, new GFNode((GFWire) gf));
                }
            }
            for (int i = 1; i < to - from; i++) {
                gf = get(from + i, where);
                if (gf.getClass().equals(GFEmpty.class)) {
                    set(from + i, where, new GFWire(true, false, true, false));
                }
                if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                    ((GFWire) gf).setNorth(true);
                    ((GFWire) gf).setSouth(true);
                    if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                        set(from + i, where, new GFNode((GFWire) gf));
                    }
                }
            }
            gf = get(to, where);
            if (gf.getClass().equals(GFEmpty.class)) {
                set(to, where, new GFWire(true, false, false, false));
            } else if (gf.getClass().equals(GFWire.class) | gf.getClass().equals(GFNode.class)) {
                ((GFWire) gf).setNorth(true);
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(to, where, new GFNode((GFWire) gf));
                }
            }
        }
    }

    /**
     * odstraní dráty a uzly
     * @param where řádek/sloupec(rozlišeno par. horizontal), kde se odebírá drát
     * @param from políčko, kde mazání začíná
     * @param to políčko, kde mazání končí TO>FROM
     * @param horizontal mazaný drát je vodorovný/svislý
     */
    public void deleteWire(int where, int from, int to, boolean horizontal) {
        GridField gf;
        if (horizontal) {
            gf = get(where, from);
            if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                ((GFWire) gf).setEast(false);
                if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                    set(where, from, new GFWire((GFNode) gf));
                }
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(where, from, new GFNode((GFWire) gf));
                }
            }
            if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                if (((Orientable) gf).isHorizontal()) {
                    set(where, from, new GFEmpty());
                }
            }
            for (int i = 1; i < to - from; i++) {
                gf = get(where, from + i);
                if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                    ((GFWire) gf).setEast(false);
                    ((GFWire) gf).setWest(false);
                    if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                        set(where, from + i, new GFWire((GFNode) gf));
                    }
                }
                if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                    if (((Orientable) gf).isHorizontal()) {
                        set(where, from + i, new GFEmpty());
                    }
                }
            }
            gf = get(where, to);
            if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                ((GFWire) gf).setWest(false);
                if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                    set(where, to, new GFWire((GFNode) gf));
                }
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(where, to, new GFNode((GFWire) gf));
                }
            }
            if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                if (((Orientable) gf).isHorizontal()) {
                    set(where, to, new GFEmpty());
                }
            }
        } else {
            gf = get(from, where);
            if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                ((GFWire) gf).setSouth(false);
                if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                    set(from, where, new GFWire((GFNode) gf));
                }
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(from, where, new GFNode((GFWire) gf));
                }
            }
            if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                if (!((Orientable) gf).isHorizontal()) {
                    set(from, where, new GFEmpty());
                }
            }
            for (int i = 1; i < to - from; i++) {
                gf = get(from + i, where);
                if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                    ((GFWire) gf).setNorth(false);
                    ((GFWire) gf).setSouth(false);
                    if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                        set(from + i, where, new GFWire((GFNode) gf));
                    }
                }
                if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                    if (!((Orientable) gf).isHorizontal()) {
                        set(from + i, where, new GFEmpty());
                    }
                }
            }
            gf = get(to, where);
            if (gf.getClass() == GFWire.class | gf.getClass() == GFNode.class) {
                ((GFWire) gf).setNorth(false);
                if (gf.getClass() == GFNode.class && !((GFNode) gf).needNode()) {
                    set(to, where, new GFWire((GFNode) gf));
                }
                if (gf.getClass().equals(GFWire.class) & ((GFWire) gf).needNode()) {
                    set(to, where, new GFNode((GFWire) gf));
                }
            }
            if (gf.getClass() == GFSource.class | gf.getClass() == GFResistor.class) {
                if (!((Orientable) gf).isHorizontal()) {
                    set(to, where, new GFEmpty());
                }
            }
        }
        clearEmptyWires();
    }

    /**
     * označí políčka jako "označená tažením"
     * @param where řádek/sloupec(rozlišeno par. horizontal), odkud se označuje
     * @param from políčko, kde mazání začíná
     * @param to políčko, kde mazání končí TO>FROM
     * @param horizontal mazaný drát je vodorovný/svislý
     */
    public void dragSelect(int where, int from, int to, boolean horizontal) {
        deselect();
        if (horizontal) {
            for (int i = 0; i < to - from + 1; i++) {
                get(where, from + i).setDragSelected(true);
            }
        } else {
            for (int i = 0; i < to - from + 1; i++) {
                get(from + i, where).setDragSelected(true);
            }
        }
    }

    /**
     * rozšíří grid
     * @param dir směr rozšíření
     * @param number počet nových řádků/sloupců
     */
    public void extend(Direction dir, int number) {
        if (dir.equals(Direction.NORTH)) {
            for (int i = 0; i < number; i++) {
                gridlines.add(0, new CircuitGridLine(getColCount()));
            }
        }
        if (dir == Direction.EAST) {
            for (int i = 0; i < number; i++) {
                for (int r = 0; r < getRowCount(); r++) {
                    gridlines.get(r).fields.add(new GFEmpty());
                }
            }
        }
        if (dir == Direction.SOUTH) {
            for (int i = 0; i < number; i++) {
                gridlines.add(new CircuitGridLine(getColCount()));
            }
        }
        if (dir == Direction.WEST) {
            for (int i = 0; i < number; i++) {
                for (int r = 0; r < getRowCount(); r++) {
                    gridlines.get(r).fields.add(0, new GFEmpty());
                }
            }
        }
    }

    /**
     * ořízne grid o prázdné sloupce a řádky, až do minimálního rozměru, přednostně shora a zleva
     */
    public void crop() {
        selected = none;
        clearEmptyWires();
        while (getRowCount() > MINGRIDSIZE & isEmpty(Direction.NORTH)) {
            gridlines.remove(0);
        }
        while (getRowCount() > MINGRIDSIZE & isEmpty(Direction.SOUTH)) {
            gridlines.remove(getRowCount() - 1);
        }
        while (getColCount() > MINGRIDSIZE & isEmpty(Direction.WEST)) {
            for (int i = 0; i < getRowCount(); i++) {
                gridlines.get(i).fields.remove(0);
            }
        }
        while (getColCount() > MINGRIDSIZE & isEmpty(Direction.EAST)) {
            int formerColCount = getColCount();
            for (int i = 0; i < getRowCount(); i++) {
                gridlines.get(i).fields.remove(formerColCount - 1);
            }
        }
    }

    /**
     * nahradí GFEmpty místo GFWires (a GFNodes) bez drátů (north,east,south a west == false)
     */
    private void clearEmptyWires() {
        for (int i = 0; i < getRowCount(); i++) {
            gridlines.get(i).clearEmptyWires();
        }
    }

    /**
     * zjistí, zda je daný sloupec/řádek prázdný
     * @param dir směr, ze kterého se vlastnost zjišťuje
     */
    private boolean isEmpty(Direction dir) {
        if (dir == Direction.NORTH) {
            return gridlines.get(0).isEmpty();
        }
        if (dir == Direction.SOUTH) {
            return gridlines.get(getRowCount() - 1).isEmpty();
        }
        if (dir == Direction.EAST) {
            for (int i = 0; i < getRowCount(); i++) {
                if (!gridlines.get(i).fields.get(getColCount() - 1).getClass().equals(GFEmpty.class)) {
                    return false;
                }
            }
        }
        if (dir == Direction.WEST) {
            for (int i = 0; i < getRowCount(); i++) {
                if (!gridlines.get(i).fields.get(0).getClass().equals(GFEmpty.class)) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * @return šířka mřížky v px
     */
    public int getWidth() {
        return getColCount() * getGRIDCONSTANT();
    }

    /**
     * @return výška mřížky v px
     */
    public int getHeight() {
        return getRowCount() * getGRIDCONSTANT();
    }

    /**
     * @return počet řádků
     */
    public int getRowCount() {
        return gridlines.size();
    }

    /**
     * @return počet sloupců
     */
    public int getColCount() {
        return gridlines.get(0).getColCount();
    }

    /**
     * @param row řádek
     * @param col sloupec
     * @return field na dané pozici
     */
    public GridField get(int row, int col) {
        return gridlines.get(row).fields.get(col);
    }

    /**
     * @param pos
     * @return field na dané pozici
     */
    public GridField get(Position pos) {
        return get(pos.row, pos.col);
    }

    /**
     * nahradí určený field novým
     * @param row řádek
     * @param col sloupec
     * @param field nový field
     */
    private void set(int row, int col, GridField field) {
        gridlines.get(row).fields.set(col, field);
    }

    /**
     * překreslí Grid do dodaného grafického kontextu pomocí Double Bufferingu
     * @param paintMarks vykreslit označení větví a uzlů?
     */
    public void paint(Graphics g, boolean paintMarks) {
        BufferedImage buf = new BufferedImage(gp.getWidth(), gp.getHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics offG = buf.getGraphics();
        offG.setColor(Color.GRAY);
        offG.fillRect(0, 0, buf.getWidth(), buf.getHeight());
        for (int row = 0; row < getRowCount(); row++) {
            for (int col = 0; col < getColCount(); col++) {
                boolean b = get(row, col).equals(getSelected());
                /*if (b) {
                System.out.print("T");
                } else {
                System.out.print("_");
                }*/
                get(row, col).paint(offG, col * getGRIDCONSTANT(), row * getGRIDCONSTANT(), getGRIDCONSTANT(), (get(row, col).equals(getSelected())), paintMarks);
            }
            //System.out.println("");
        }
        //System.out.println("");
        g.drawImage(buf, 0, 0, null);
    }

    /**
     * zruší veškeré označení políček gridu
     */
    public void deselect() {
        setSelected(none);
        deDragSelect();
    }

    /**
     * zruší označení políček tažením (modré)
     */
    public void deDragSelect() {
        for (int c = 0; c < getColCount(); c++) {
            for (int r = 0; r < getRowCount(); r++) {
                get(r, c).setDragSelected(false);
            }
        }
    }

    /**
     * označí všechna pole jako nezvýrazněná
     */
    public void unHighlight() {
        for (int c = 0; c < getColCount(); c++) {
            for (int r = 0; r < getRowCount(); r++) {
                get(r, c).setHighlighted(false);
            }
        }
    }

    /**
     * nahradí označený úsek drátu zdrojem
     * @param voltage napětí nového zdroje
     */
    public void setSource(double voltage) {
        GFSource source = new GFSource((GFWire) getSelected(), voltage);
        set(selectedY, selectedX, source);
        setSelected(source);
    }

    /**
     * nahradí označený úsek drátu rezistorem
     * @param resistance odpor zdroje
     */
    public void setResistor(double resistance) {
        GFResistor resistor = new GFResistor((GFWire) getSelected(), resistance);
        set(selectedY, selectedX, resistor);
        setSelected(resistor);
    }

    /**
     * přidá na označené políčko uzel
     */
    public void setNode() {
        GFNode node = new GFNode((GFWire) getSelected());
        set(selectedY, selectedX, node);
        setSelected(node);
    }

    /**
     * odstraní uzel s označeného políčka
     */
    public void removeNode() {
        GFWire wire = new GFWire((GFNode) getSelected());
        set(selectedY, selectedX, wire);
        setSelected(wire);
    }

    /**
     * nahradí označenou komponentu rovným úsekem drátu
     */
    public void removeComponent() {
        if (getSelected().getClass() == GFResistor.class) {
            GFWire wire = new GFWire((GFResistor) getSelected());
            set(selectedY, selectedX, wire);
            setSelected(wire);
        }
        if (getSelected().getClass() == GFSource.class) {
            GFWire wire = new GFWire((GFSource) getSelected());
            set(selectedY, selectedX, wire);
            setSelected(wire);
        }

    }

    /**
     * změní napětí/odpor označeného zdroje/rezistoru
     * @param value nová hodnota komponenty
     */
    public void changeComponentValue(double value) {
        if (getSelected().getClass() == GFResistor.class) {
            ((GFResistor) getSelected()).setResistance(value);
        }
        if (getSelected().getClass() == GFSource.class) {
            ((GFSource) getSelected()).setVoltage(value);
        }

    }

    /**
     * zmeni polaritu zdroje
     */
    public void changeSourcePolarity() {
        ((GFSource) getSelected()).changePolarity();
    }

    /**
     * označí příslušnou část obvodu za zvýrazněnou a přidělí polím barvy zvýraznění
     * @param part zvýrazněná část obvodu
     */
    public void highlight(Highlightable part, Graphics g) {
        deselect();
        unHighlight();
        Iterator<Position> it = part.getPositions().iterator();
        Position pos;
        double green = 0;
        double greenInc = 0;
        boolean loop = false;
        if (part.getClass() == Loop.class) {
            //System.out.println("\nLoop Highlighted");
            loop = true;
            double d = part.getPositions().size() - 1;
            greenInc = 1 / d;
            //System.out.println("GreenInc:"+greenInc);
        }
        while (it.hasNext()) {
            pos = it.next();
            GridField field = get(pos.row, pos.col);
            field.setHighlighted(true);
            field.setHighlightColor();
            //System.out.println(field);
            if (loop) {
                Color c = field.setHighlightColor(green);
                //System.out.println("Highlight position "+pos.row+" "+pos.col);
                //System.out.println("\n"+ green);
                //System.out.println("Color:"+c);
                green += greenInc;
            }
        }
        paint(g, true);
    }

    /**
     * označí uzly po řádkách písmeny A, B, C...
     */
    public void setMarks() {
        GridField gf;
        char c = 'A';
        for (int row = 0; row < getRowCount(); row++) {
            for (int col = 0; col < getColCount(); col++) {
                gf = get(row, col);
                if (gf.getClass().equals(GFNode.class)) {
                    ((GFNode) gf).setMark(Character.toString(c));
                    c++;
                }
            }
        }
    }

    /**
     * @return the GRIDCONSTANT
     */
    public int getGRIDCONSTANT() {
        return GRIDCONSTANT;
    }

    /**
     * @param GRIDCONSTANT the GRIDCONSTANT to set
     */
    public void setGRIDCONSTANT(int GRIDCONSTANT) {
        this.GRIDCONSTANT = GRIDCONSTANT;
    }

    /**
     * @return the lastDragSelected
     */
    public GridField getLastDragSelected() {
        return lastDragSelected;
    }

    /**
     * @param lastDragSelected the lastDragSelected to set
     */
    public void setLastDragSelected(GridField lastDragSelected) {
        this.lastDragSelected = lastDragSelected;
    }

    /**
     * @return the selected
     */
    public GridField getSelected() {
        return selected;
    }

    /**
     * @param selected the selected to set
     */
    public void setSelected(GridField selected) {
        this.selected = selected;
    }

    /**
     * @param selectedX the selectedX to set
     */
    public void setSelectedX(int selectedX) {
        this.selectedX = selectedX;
    }

    /**
     * @param selectedY the selectedY to set
     */
    public void setSelectedY(int selectedY) {
        this.selectedY = selectedY;
    }

    /**
     * @return the MINFIELDSIZE
     */
    public int getMINFIELDSIZE() {
        return MINFIELDSIZE;
    }

    /**
     * zkontroluje zpracovatelnost mřížky, při nalezení chyby hodí výjimku
     */
    public void canProcess() throws CircuitNotClosedException, NoCircuitException, NoSourceException, NoResistorException {
        GridField gf;
        int sources = 0;
        int nodes = 0;
        int resistors = 0;
        for (int row = 0; row < getRowCount(); row++) {
            for (int col = 0; col < getColCount(); col++) {
                gf = get(row, col);
                if (gf.getClass().equals(GFWire.class)) {
                    if (((GFWire) gf).numberOfWires() == 1) {
                        throw new CircuitNotClosedException();
                    }
                }
                if (gf.getClass().equals(GFSource.class)) {
                    sources++;
                }
                if (gf.getClass().equals(GFResistor.class)) {
                    resistors++;
                }
                if (gf.getClass().equals(GFNode.class)) {
                    ((GFNode) gf).resetCurrents();
                    nodes++;
                }
            }
        }
        if (nodes == 0) {
            throw new NoCircuitException();
        }
        if (sources == 0) {
            throw new NoSourceException();
        }
        if (resistors == 0) {
            throw new NoResistorException();
        }
    }

    /**
     * @return the MAXFIELDSIZE
     */
    public int getMAXFIELDSIZE() {
        return MAXFIELDSIZE;
    }
}
