package PresentationLayer.Grid;

import ApplicationLayer.NumOp;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

/**
 * políčko zdroje
 * @author Jakub Hrncir (2010)
 */
public class GFSource extends GridField implements Orientable{

    private boolean horizontal;
    private double voltage;
    private boolean polarity;
    
    public GFSource(GFWire wire, double voltage) {
        horizontal = (wire.isHorizontal()) ? true : false;
        polarity = true;
        this.voltage = voltage;
    }

    @Override
    public void paint(Graphics g, int x1, int y1, int GC, boolean selected, boolean paintMarks) {
        g.setColor((selected) ? COLOR_SELECT : COLOR_NOSELECT);
        if (dragSelected) {
            g.setColor(COLOR_DRAGSELECT);
        }
        if (highlighted) {
            g.setColor(color_highlight);
        }
        g.fillRect(x1, y1, GC, GC);
        g.setColor(COLOR_BORDER);
        g.drawRect(x1, y1, GC, GC);
        g.setColor(COLOR_LINE);
        if (isHorizontal()) {
            g.drawLine(x1, y1 + (GC / 2), x1 + (GC / 4), y1 + (GC / 2));
            g.drawOval(x1 + GC / 4, y1 + 7 * GC / 16, GC / 8, GC / 8);
            g.drawOval(x1 + GC - GC / 4 - GC / 8, y1 + 7 * GC / 16, GC / 8, GC / 8);
            g.drawLine(x1 + GC - (GC / 4), y1 + (GC / 2), x1 + GC, y1 + (GC / 2));
            g.setFont(new Font("Arial", Font.PLAIN, GC / 5));
            g.drawString(NumOp.writeDouble(voltage), x1 + GC / 8, y1 + GC / 4);
            g.setFont(new Font("Arial", Font.PLAIN, GC / 3));
            g.drawString((getPolarity()) ? "+" : "–", x1 + GC / 5, y1 + 4 * GC / 5);
            g.drawString((getPolarity()) ? "–" : "+", x1 + 3 * GC / 5, y1 + 4 * GC / 5);
        } else {
            g.drawLine(x1 + (GC / 2), y1, x1 + (GC / 2), y1 + (GC / 4));
            g.drawOval(x1 + 7 * GC / 16, y1 + GC / 4, GC / 8, GC / 8);
            g.drawOval(x1 + 7 * GC / 16, y1 + GC - GC / 4 - GC / 8, GC / 8, GC / 8);
            g.drawLine(x1 + (GC / 2), y1 + GC - (GC / 4), x1 + (GC / 2), y1 + GC);
            g.setFont(new Font("Arial", Font.PLAIN, GC / 5));
            g.setColor(COLOR_LINE);
            g.drawString(NumOp.writeDouble(voltage), x1 + GC / 8, y1 + (23 * GC / 40));
            g.setFont(new Font("Arial", Font.PLAIN, GC / 3));
            g.drawString((getPolarity()) ? "+" : "–", x1 + GC / 5, y1 + 1 * GC / 3);
            g.drawString((getPolarity()) ? "–" : "+", x1 + GC / 5, y1 + 6 * GC / 7);
        }
    }

    @Override
    public boolean isCleared() {
        return false;
    }
    /**
     * nastaví barvu zvýraznění
     * @param green musí být mezi 0-1, 0 je nejsvětlejší, 1 nejtmavší zelená
     */
    @Override
    public Color setHighlightColor(double green) {
        color_highlight = new Color((int) ((1 - green) * COLOR_CONST), 255, (int) ((1 - green) * COLOR_CONST));
        return new Color((int) ((1 - green) * COLOR_CONST), 255, (int) ((1 - green) * COLOR_CONST));
    }
    @Override
    public void setHighlightColor() {
        color_highlight = COLOR_HIGHLIGHT;
    }

    /**
     * @return the voltage
     */
    public double getVoltage() {
        return voltage;
    }

    /**
     * @param voltage the voltage to set
     */
    public void setVoltage(double voltage) {
        this.voltage = voltage;
    }

    /**
     * @return the horizontal
     */
    public boolean isHorizontal() {
        return horizontal;
    }

    /**
     * @return the polarity
     */
    public boolean getPolarity() {
        return polarity;
    }

    public void changePolarity() {
        polarity = !polarity;
    }
}
