package PresentationLayer.Grid;

import ApplicationLayer.Direction;
import java.awt.Graphics;
import java.util.ArrayList;

/**
 * políčko prázdného vodiče
 * @author Jakub Hrncir (2010)
 */
public class GFWire extends GridField {

    protected boolean north;
    protected boolean east;
    protected boolean south;
    protected boolean west;

    public GFWire(boolean north, boolean east, boolean south, boolean west) {
        this.north = north;
        this.east = east;
        this.south = south;
        this.west = west;
    }

    public GFWire(GFNode node) {
        this(node.north, node.east, node.south, node.west);
    }

    public GFWire(GFResistor resistor) {
        this(!(resistor.isHorizontal()), (resistor.isHorizontal()), !(resistor.isHorizontal()), (resistor.isHorizontal()));
    }

    public GFWire(GFSource source) {
        this(!(source.isHorizontal()), (source.isHorizontal()), !(source.isHorizontal()), (source.isHorizontal()));
    }

    @Override
    public void paint(Graphics g, int x1, int y1, int GC, boolean selected, boolean paintMarks) {
        g.setColor((selected) ? COLOR_SELECT : COLOR_NOSELECT);
        if (dragSelected) {
            g.setColor(COLOR_DRAGSELECT);
        }
        if (highlighted) {
            g.setColor(color_highlight);
        }
        g.fillRect(x1, y1, GC, GC);
        g.setColor(COLOR_BORDER);
        g.drawRect(x1, y1, GC, GC);
        g.setColor(COLOR_LINE);
        if (isNorth()) {
            g.drawLine(x1 + (GC / 2), y1, x1 + (GC / 2), y1 + (GC / 2));
        }
        if (isEast()) {
            g.drawLine(x1 + (GC / 2), y1 + (GC / 2), x1 + GC, y1 + (GC / 2));
        }
        if (isSouth()) {
            g.drawLine(x1 + (GC / 2), y1 + (GC / 2), x1 + (GC / 2), y1 + GC);
        }
        if (isWest()) {
            g.drawLine(x1, y1 + (GC / 2), x1 + (GC / 2), y1 + (GC / 2));
        }
    }

    public boolean isCleared() {
        if (north) {
            return false;
        }
        if (east) {
            return false;
        }
        if (south) {
            return false;
        }
        if (west) {
            return false;
        }
        return true;
    }

    public boolean isHorizontal() {
        return (west == true & east == true & north == false & south == false);
    }

    public boolean isVertical() {
        return (west == false & east == false & north == true & south == true);
    }

    /**
     * 
     * @return true pokud lze umístit componentu (jedná se o rovný úsek drátu, nekříží se)
     */
    public boolean canPutComponent() {
        return (isHorizontal() | isVertical());
    }

    protected int numberOfWires() {
        int i = 0;
        if (north) {
            i++;
        }
        if (east) {
            i++;
        }
        if (south) {
            i++;
        }
        if (west) {
            i++;
        }
        return i;
    }

    public boolean needNode() {
        return (numberOfWires() == 3);
    }

    public Direction otherDir(Direction dir) {
        ArrayList<Direction> dirs = getDirs();
        if (dirs.size() == 4) {
            return dir.invert();
        }
        if (dirs.size() != 2) {
            System.out.println("otherDir Error (GFWire)");
            return null;
        }
        return (dirs.get(0).ordinal() == dir.ordinal()) ? dirs.get(1) : dirs.get(0);
    }

    public ArrayList<Direction> getDirs() {
        ArrayList<Direction> dirs = new ArrayList<Direction>(4);
        if (north) {
            dirs.add(Direction.NORTH);
        }
        if (east) {
            dirs.add(Direction.EAST);
        }
        if (south) {
            dirs.add(Direction.SOUTH);
        }
        if (west) {
            dirs.add(Direction.WEST);
        }
        return dirs;
    }

    protected boolean isDirection(Direction dir) {
        if (dir == Direction.NORTH & north == true) {
            return true;
        }
        if (dir == Direction.EAST & east == true) {
            return true;
        }
        if (dir == Direction.SOUTH & south == true) {
            return true;
        }
        if (dir == Direction.WEST & west == true) {
            return true;
        }
        return false;
    }

    /**
     * @return the north
     */
    public boolean isNorth() {
        return north;
    }

    /**
     * @param north the north to set
     */
    public void setNorth(boolean north) {
        this.north = north;
    }

    /**
     * @return the east
     */
    public boolean isEast() {
        return east;
    }

    /**
     * @param east the east to set
     */
    public void setEast(boolean east) {
        this.east = east;
    }

    /**
     * @return the south
     */
    public boolean isSouth() {
        return south;
    }

    /**
     * @param south the south to set
     */
    public void setSouth(boolean south) {
        this.south = south;
    }

    /**
     * @return the west
     */
    public boolean isWest() {
        return west;
    }

    /**
     * @param west the west to set
     */
    public void setWest(boolean west) {
        this.west = west;
    }
}
