package PresentationLayer;

import PresentationLayer.Forms.MFState;
import PresentationLayer.Forms.MainForm;
import PresentationLayer.Grid.CircuitGrid;
import PresentationLayer.Grid.GFResistor;
import PresentationLayer.Grid.GFSource;
import PresentationLayer.Grid.GFWire;
import PresentationLayer.Grid.GridField;
import java.awt.Dimension;
import java.awt.Graphics;
import javax.swing.JPanel;

/**
 *
 * @author Jakub Hrncir (2010)
 */
public class GridPannel extends JPanel {

    private CircuitGrid cg;
    private MainForm mf;

    /**
     * panel obsahující nákres obvodu
     */
    public GridPannel(MainForm mf) {
        this.mf = mf;
        cg = CircuitGrid.self;
    }

    /**
     * vykreslí mřížku i její okolí
     * @param width šířka scrollpane
     * @param height výška scrollpane
     */
    public void paintWholeComponent(Graphics g, int width, int height) {
        cg.gp = this;
        setPreferredSize(new Dimension(cg.getWidth(), cg.getHeight()));
        paintComponent(g);
    }

    /**
     * vykreslí mřížku
     * ! rozměry musí být zachovány
     */
    @Override
    public void paintComponent(Graphics g) {
        //super.paintComponent(g); toto způsobuje blikání
        cg.paint(g, mf.needMarks());
    }

    /**
     * ošetří kliknutí
     * @param x mouse click x (px)
     * @param y mouse click y (px)
     */
    public void mouseClick(int x, int y, int mouseButton) {
        int GC = cg.getGRIDCONSTANT();
        if ((x < cg.getWidth()) & (y < cg.getHeight())) {
            //System.out.println("Click");
            GridField gf = cg.get(y / GC, x / GC);
            if (!gf.equals(cg.getSelected())) {
                cg.setSelected(cg.get(y / GC, x / GC));
                cg.setSelectedX(x / GC);
                cg.setSelectedY(y / GC);
            } else {
                if (mouseButton == 3) {
                    gf = cg.getSelected();
                    if (gf.getClass() == GFWire.class) {
                        if (((GFWire) gf).canPutComponent()) {
                            cg.setResistor(mf.getComponentValue());
                        }
                    }
                    if (gf.getClass() == GFResistor.class) {
                        cg.removeComponent();
                        cg.setSource(mf.getComponentValue());
                    }
                    if (gf.getClass() == GFSource.class) {
                        cg.removeComponent();
                    }
                }
            }
            //System.out.println("sel: " + (y / GC) + " " + (x / GC));
        }
    }

    /**
     * ošetří mouse drag (drag musí skončit buď ve stejném řádku, nebo sloupci)
     * pokud drag skončí ve stejné buňce, vyhodnotí se jako kliknutí
     * @param x1 mouse pressed x (px)
     * @param y1 mouse pressed y (px)
     * @param x2 mouse released x (px)
     * @param y2 mouse released y (px)
     */
    public void mouseDrag(int x1, int y1, int x2, int y2, int button, MFState state) {
        int GC = cg.getGRIDCONSTANT();
        if ((x1 < cg.getWidth()) & (y1 < cg.getHeight()) & (x2 < cg.getWidth()) & (y2 < cg.getHeight())) {
            if ((y1 / GC == y2 / GC) & (x1 / GC == x2 / GC)) {
                if (button == 1 | button == 3) {
                    cg.deDragSelect();
                    mouseClick(x2, y2, button);
                    return;
                }
            }
            if (state.ordinal() > 0) {
                return;
            }
            if ((y1 / GC == y2 / GC) & (x1 / GC != x2 / GC)) {
                if (button == 1) {
                    cg.drawWire(y1 / GC, Math.min(x1 / GC, x2 / GC), Math.max(x1 / GC, x2 / GC), true);
                }
                if (button == 3) {
                    cg.deleteWire(y1 / GC, Math.min(x1 / GC, x2 / GC), Math.max(x1 / GC, x2 / GC), true);
                }

            }
            if ((y1 / GC != y2 / GC) & (x1 / GC == x2 / GC)) {
                if (button == 1) {
                    cg.drawWire(x1 / GC, Math.min(y1 / GC, y2 / GC), Math.max(y1 / GC, y2 / GC), false);
                }
                if (button == 3) {
                    cg.deleteWire(x1 / GC, Math.min(y1 / GC, y2 / GC), Math.max(y1 / GC, y2 / GC), false);
                }
            }
        }
        cg.deselect();
    }

    /**
     * ošetří probíhající mouse dragging (drag musí být buď ve stejném řádku, nebo sloupci)
     * @param x1 mouse pressed x (px)
     * @param y1 mouse pressed y (px)
     * @param x2 mouse now x (px)
     * @param y2 mouse now y (px)
     * return zda se změnil vzhled gridu (zda je potřeba vykreslení)
     */
    public boolean mouseDraging(int x1, int y1, int x2, int y2) {
        boolean changed = false;
        int GC = cg.getGRIDCONSTANT();
        if ((x1 < cg.getWidth()) & (y1 < cg.getHeight()) & (x2 < cg.getWidth()) & (y2 < cg.getHeight())) {
            if ((y1 / GC == y2 / GC) & (x1 / GC != x2 / GC) & !(cg.get(y2 / GC, x2 / GC).equals(cg.getLastDragSelected()))) {
                cg.dragSelect(y1 / GC, Math.min(x1 / GC, x2 / GC), Math.max(x1 / GC, x2 / GC), true);
                cg.setLastDragSelected(cg.get(y2 / GC, x2 / GC));
                changed = true;
            }
            if ((y1 / GC != y2 / GC) & (x1 / GC == x2 / GC) & !(cg.get(y2 / GC, x2 / GC).equals(cg.getLastDragSelected()))) {
                cg.dragSelect(x1 / GC, Math.min(y1 / GC, y2 / GC), Math.max(y1 / GC, y2 / GC), false);
                cg.setLastDragSelected(cg.get(y2 / GC, x2 / GC));
                changed = true;
            }
        }
        return changed;
    }

    /**
     * ošetří zvětšení
     * @return má být povoleno další použití této metody
     */
    public boolean zoomIn() {
        if (cg.getGRIDCONSTANT() < cg.getMAXFIELDSIZE()) {
            cg.setGRIDCONSTANT(cg.getGRIDCONSTANT() + 10);
        }
        return (cg.getGRIDCONSTANT() < cg.getMAXFIELDSIZE()) ? true : false;
    }

    /**
     * ošetří zmenšení
     * @return má být povoleno další použití této metody
     */
    public boolean zoomOut() {
        if (cg.getGRIDCONSTANT() > cg.getMINFIELDSIZE()) {
            cg.setGRIDCONSTANT(cg.getGRIDCONSTANT() - 10);
        }
        return (cg.getGRIDCONSTANT() > cg.getMINFIELDSIZE()) ? true : false;
    }
}
